import Async, { CustomSignalClass, Signal } from "typescene-async";
import { ActivityResourcePath } from "./ActivityPath";
/** Represents a UI activity */
export declare class Activity extends Async.ObservableObject {
    /** Get an instance of this activity class, can be overridden to alter behavior when e.g. pushing a parent activity by class; by default simply constructs the instance without parameters */
    static getInstance(): Activity;
    /** Display title of this activity (observable) */
    title: string | undefined;
    /** Object that represents the internal state of the activity; this is a sealed observable object, never null or undefined, and setting this to a plain object ({...}) will create an ObservableObject instance (in practice, initialize all properties as activity.state = { x: ... } first to make properties observable); it is recommended to keep properties on the state object instead of on the Activity instance itself, and provide public access through the derived Activity's methods (e.g. to view instances) */
    protected state: {};
    /** The parent activity, if any; the activity manager will ensure the given activity (or an instance of given class) exists in its history stack when activating this activity, or insert it before */
    parentActivity: Activity | typeof Activity | null;
    /** Signal that is emitted before this activity is pushed to foreground, if any handler throws an error then the activity is not activated */
    readonly Activating: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted after this activity is pushed to foreground */
    readonly Activated: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted before another activity is pushed to foreground, if any handler throws an error then the activity is not suspended */
    readonly Suspending: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted when this activity is no longer in foreground */
    readonly Suspended: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted before this activity is put back in foreground, if any handler throws an error then the activity is not suspended */
    readonly Resuming: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted when this activity is back in foreground */
    readonly Resumed: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Signal that is emitted after this activity is removed from history */
    readonly Deactivated: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
}
/** Represents an activity pertaining to a resource (URL path) */
export declare class ResourceActivity extends Activity {
    /** Create an activity with given resource path */
    constructor(path?: ActivityResourcePath | string);
    /** The path (URL/hash segment) that is associated with this activity */
    resourcePath: ActivityResourcePath | string | undefined;
}
/** Represents a transition from one foreground activity to another */
export interface ActivityTransition {
    /** The activity stack that is transitioning */
    activityManager: ActivityManager;
    /** The activity that is currently in the foreground, if any */
    from: Activity;
    /** The new foreground activity, if any */
    to: Activity;
    /** The stack operation being performed: push, replace, or pop */
    operation: ActivityTransition.Operation;
    /** The previous transition that is part of the same operation (pop/push/pop/up) */
    previous: ActivityTransition;
}
export declare namespace ActivityTransition {
    /** Operation type that triggered a transition */
    enum Operation {
        /** Push operation */
        Push = 0,
        /** Replace operation */
        Replace = 1,
        /** Pop operation */
        Pop = 2,
    }
}
/** Represents activities that have been activated (activity history) */
export declare class ActivityManager {
    /** Add an activity to the foreground asynchronously, does nothing if given activity was already in the foreground; returns Promise that resolves to the completed transition */
    push(activity: Activity): PromiseLike<ActivityTransition>;
    /** Replace the current activity asynchronously (throws error if none), or remove current activity if given activity was already directly below current activity in the activity stack; returns Promise that resolves to the completed transition */
    replace(activity: Activity): PromiseLike<ActivityTransition>;
    /** Remove the current foreground activity (go back) asynchronously, returns Promise that resolves to the completed transition */
    pop(): PromiseLike<ActivityTransition>;
    /** Remove foreground activities until activity of given type is in the foreground; returns Promise that resolves to activity, or null if there was no matching activity on the stack */
    up(ActivityClass: typeof Activity): PromiseLike<Activity | null>;
    /** Reload state using given history ID, if possible (i.e. not yet deactivated relevant activities in the meantime); returns a promise that resolves when the state has been reached */
    restoreHistoryState(historyID: string): PromiseLike<void>;
    /** Get an ID that represents the current state, for use with reset(...) */
    getHistoryState(): string;
    /** Get the activity closest to the foreground of the given type, if any (excluding foreground activity itself, and before given activity in second parameter, if any) */
    getParent(ActivityClass: typeof Activity, before?: Activity): Activity | null;
    /** Returns true if the stack contains given activity */
    contains(activity: Activity): boolean;
    /** The current foreground activity (top of stack, if any; observable) */
    readonly top: Activity | null;
    /** The number of activities on the stack (observable) */
    readonly length: number;
    /** The title of the topmost activity that has a title defined (observable) */
    readonly title: string | undefined;
    /** Signal that is emitted when a transition is happening (after Activating/Resuming/Suspending but before Activated/Resumed) */
    readonly Transition: CustomSignalClass<ActivityTransition, Signal<ActivityTransition>>;
    /** Helper method to find parent activity/ies for given activity; returns an array of activities to be pushed (including activity) */
    private _findParents(activity, excludeTop?);
    /** Helper method for sending signals and creating promise */
    private _processTransition(to, operation);
    private _topIndex;
    private _stack;
    private _ids;
    private _transitionP;
}
export default Activity;
