import Async from "typescene-async";
import Style from "../Style";
import { MouseEventSignal, KeyEventSignal, FocusEventSignal, DragEventSignal, MouseHandler, KeyHandler, FocusHandler, DragHandler } from "./DOMEvents";
import ComponentRenderOutput, { ComponentRenderSignal, RenderHandler } from "./ComponentRenderOutput";
import { ComponentFactory, ValueOrObservable } from "./ComponentFactory";
/** Component base class for every piece of the UI on screen */
export declare abstract class Component extends Async.ObservableObject {
    /** Component factory/initializer, applies given (observable) values */
    static with: <T extends Component, InitializerT extends Component.Initializer>(values: InitializerT) => ComponentFactory<T>;
    /** Initializes an existing instance with given values; returns this */
    initializeWith: (values: Component.Initializer) => this;
    /** Identifier (optional), can be set on sub components to find them through .getComponentById(...) */
    id: string;
    /** Encapsulation of inline CSS style (observed) */
    style: Style;
    /** Default class name(s), defined on class prototype; not observed */
    protected _initialStyleClassName: string;
    /** Promise that resolves to the rendered output when ready */
    private _renderedOut;
    private _resolve_renderedOut;
    /** Rendered output for this component (observable) */
    readonly out: ComponentRenderOutput<HTMLElement>;
    /** [implementation] Flag that can be used for duck typing of renderable components */
    readonly isRenderableComponent: boolean;
    /** Becomes true the first time this component is rendered (i.e. when .out is set; not observable, use Rendered signal instead) */
    readonly isRendered: boolean;
    /** Returns an array of directly contained UI components (observable) */
    getChildren(): Component[];
    /** Returns the nearest matching child element with given ID, if any */
    getComponentById(id: string): Component | null;
    /** Returns all matching child elements of given class (recursively) */
    getComponentsByType<C extends Component>(componentClass: {
        new (...p: any[]): C;
    }): C[];
    /** Returns an object containing all current values of input elements (observable) */
    getFormValues(result?: {}): any;
    /** Sets all input values by element name (will not trigger observable) */
    setFormValues(values: any): void;
    /** Signal emitted after updating DOM (render) */
    readonly Rendered: ComponentRenderSignal<HTMLElement>;
    /** Signal emitted on click (DOM) */
    readonly Click: MouseEventSignal;
    /** Signal emitted on mouse-down (DOM) */
    readonly Press: MouseEventSignal;
    /** Signal emitted on mouse-over (DOM) for this element or a child element */
    readonly MouseOver: MouseEventSignal;
    /** Signal emitted on mouse-out (DOM) for this element or a child element */
    readonly MouseOut: MouseEventSignal;
    /** Signal emitted when this element or a child element receives focus */
    readonly Focus: FocusEventSignal;
    /** Signal emitted when this element or a child element loses focus */
    readonly Blur: FocusEventSignal;
    /** Signal emitted when a key is pressed (while focused) */
    readonly KeyDown: KeyEventSignal;
    /** Signal emitted when this element or a child element commences a drag operation (e.g following a call to Drag.start) */
    readonly DragStart: DragEventSignal;
    /** Signal emitted when a drag operation hovers over this element or a child element */
    readonly DragOver: DragEventSignal;
    /** Signal emitted when a drag operation leaves this element or a child element */
    readonly DragOut: DragEventSignal;
    /** Signal emitted when this element or a child element is the target of a drop after a drag operation */
    readonly DragDrop: DragEventSignal;
}
export declare namespace Component {
    /** Initializer for .with({ ... }) */
    interface Initializer {
        /** Identifier, used to add a component reference to the base component (on which .with(...) or .initializeWith(...) was called) as a property with given identifier */
        id?: string;
        /** Property initializer */
        style?: ValueOrObservable<Style | Style.StyleSet>;
        /** Signal initializer: method name or handler */
        Rendered?: string | RenderHandler<HTMLElement>;
        /** Signal initializer: method name or handler */
        Click?: string | MouseHandler;
        /** Signal initializer: method name or handler */
        Press?: string | MouseHandler;
        /** Signal initializer: method name or handler */
        MouseOver?: string | MouseHandler;
        /** Signal initializer: method name or handler */
        MouseOut?: string | MouseHandler;
        /** Signal initializer: method name or handler */
        Focus?: string | FocusHandler;
        /** Signal initializer: method name or handler */
        Blur?: string | FocusHandler;
        /** Signal initializer: method name or handler */
        KeyDown?: string | KeyHandler;
        /** Signal initializer: method name or handler */
        DragStart?: string | DragHandler;
        /** Signal initializer: method name or handler */
        DragOver?: string | DragHandler;
        /** Signal initializer: method name or handler */
        DragOut?: string | DragHandler;
        /** Signal initializer: method name or handler */
        DragDrop?: string | DragHandler;
    }
}
export default Component;
