import Async from "typescene-async";
import Component from "./Component";
import TextLabelFactory from "./TextLabelFactory";
export declare type ValueOrObservable<T> = T | Async.ObservableValue<T>;
/** Component factory class: constructor creates and initializes a Component */
export interface ComponentFactory<T extends Component> {
    /** Contruct the component with all data contained in this factory */
    new (): T;
    /** Initializes more properties using given values; returns this */
    with(initializer: any): this;
    /** Initializes a property asynchronously using the value of an observable property from the base component (on which .with(...) or .initializeWith(...) was called), by property name/path, and using given transformation function, if any; useful for creating reusable component factories; returns this */
    withProperty(targetName: string, sourcePath?: string, f?: (v: any) => any): this;
    /** Registers a callback to be invoked for every instance after initialization; callbacks are invoked with the created UI component as its first argument, and a base component (target for properties set from .id) as its second argument; returns this */
    then(callback: (component: T, factoryBase: Component) => void): this;
    /** Creates an instance (same as using constructor, but can be chained) */
    create(): T;
}
export declare namespace ComponentFactory {
    /** A factory initializer element (e.g. control element) */
    type SpecElt = ValueOrObservable<ComponentFactory<Component> | TextLabelFactory | Component | typeof Component>;
    /** A factory initializer element (e.g. control element) or table content */
    type SpecEltTCol = ValueOrObservable<ComponentFactory<Component> | TextLabelFactory | Component | typeof Component | string | number>;
    /** A list of factory initializer elements (e.g. row) */
    type SpecList = ValueOrObservable<(SpecElt | null)[]>;
    /** A list of factory initializer elements (e.g. row) or table content */
    type SpecListTCol = ValueOrObservable<(SpecEltTCol | null)[]>;
    /** A list of factory initializer elements, or a single element */
    type SpecEltOrList = ValueOrObservable<SpecElt | (SpecElt | null)[]>;
    /** A list of factory initializer elements, or a single element; or table content */
    type SpecEltOrListTCol = ValueOrObservable<SpecEltTCol | (SpecEltTCol | null)[]>;
    /** A list of (lists of) factory initializer elements */
    type SpecList2 = ValueOrObservable<(SpecEltOrList | null)[]>;
    /** A list of (lists of) factory initializer elements or table content */
    type SpecList2TCol = ValueOrObservable<(SpecEltOrListTCol | null)[]>;
    /** Class decorator that uses given initializer specification to initialize every component instance asynchronously (after running the component constructor) */
    interface Decorator<InitializerT extends Component.Initializer, ComponentT extends Component> {
        (initializer: InitializerT): {
            (target: {
                with(initializer: any): ComponentFactory<ComponentT>;
            }): void;
        };
    }
}
export default ComponentFactory;
