import { CustomSignalClass, Signal } from "typescene-async";
import Activity from "../../Activity";
import { ActivityResourcePath } from "../../ActivityPath";
import Style from "../../Style";
import { Container, Menu } from "../";
import ControlElement from "./ControlElement";
import { ComponentFactory, ValueOrObservable } from "../ComponentFactory";
import TextLabelFactory from "../TextLabelFactory";
import { ActionHandler, MouseHandler } from "../DOMEvents";
/** Represents a button control */
export declare class Button extends ControlElement {
    /** Create a button control element */
    constructor(label?: string | TextLabelFactory, icon?: string);
    /** Initialize a button control factory with given values */
    static with: (values: Button.Initializer) => ComponentFactory<Button>;
    /** Initialize a button control factory with given label and handler */
    static withLabel(label: ValueOrObservable<string> | TextLabelFactory, clickHandler?: string | MouseHandler): ComponentFactory<Button>;
    /** Initialize a button control factory with given icon and handler */
    static withIcon(icon: ValueOrObservable<string>, clickHandler?: string | MouseHandler): ComponentFactory<Button>;
    /** Initialize with given (observable) values; returns this */
    initializeWith: (values: Button.Initializer) => this;
    /** Button label (observed) */
    label: string;
    /** Optional icon (see Text:icon; observed) */
    icon: string | undefined;
    /** Optional icon behind the text (observed) */
    iconAfter: string | undefined;
    /** Space reserved for icon (rem units), if > 0 (observed) */
    remGutter: number | undefined;
    /** Optional badge text (see Text:badge; observed) */
    badge: string;
    /** Tooltip text (title attribute; observed) */
    tooltipTitle: string;
    /** Disabled state (observed) */
    disabled: boolean;
    /** Set to true to enable Bootstrap style "active" class (depressed) */
    active: boolean;
    /** Set to true to enable Bootstrap style "primary" class */
    primary: boolean;
    /** Used with Bootstrap button classes where a "secondary/default" type is needed if none else is specified; set to true by default, overridden by .primary property */
    secondary: boolean;
    /** Dropdown menu options to be displayed when this button is clicked (optional); defaults to null, set to array or ObservableArray to enable */
    dropdown: Menu.Option[] | null;
    /** Popover container class to be displayed when this button is clicked (optional); defaults to null, set to a container class reference to enable */
    popover: typeof Container | null;
    /** Set to true to open popover above (vertically) this button */
    popoverAbove: boolean;
    /** Title of the popover with contents defined by popover property (optional); no title is displayed if this string is undefined or empty */
    popoverTitle: string;
    /** Path to activity/URL that is started when this button is clicked (optional) */
    href: ActivityResourcePath | string | undefined;
    /** Activity that is started when this button is clicked (optional) */
    activity: Activity | typeof Activity | undefined;
    /** Set to false to expand horizontally within row (observed) */
    shrinkwrap: boolean;
    private _observeState;
    /** Encapsulation of button element style (observed) */
    style_button: Style;
    /** Signal emitted when a dropdown option has been clicked */
    readonly DropdownClicked: CustomSignalClass<string | number, Signal<string | number>>;
    protected focus(): void;
    protected blur(): void;
    private _btnElement;
}
export declare namespace Button {
    /** Initializer for .with({ ... }) */
    interface Initializer extends ControlElement.Initializer {
        /** Property initializer */
        label: ValueOrObservable<string> | TextLabelFactory;
        /** Property initializer */
        icon?: ValueOrObservable<string>;
        /** Property initializer */
        iconAfter?: ValueOrObservable<string>;
        /** Property initializer */
        remGutter?: ValueOrObservable<number>;
        /** Property initializer */
        badge?: ValueOrObservable<string> | TextLabelFactory;
        /** Property initializer */
        tooltipTitle?: ValueOrObservable<string> | TextLabelFactory;
        /** Property initializer */
        disabled?: ValueOrObservable<boolean>;
        /** Property initializer */
        active?: ValueOrObservable<boolean>;
        /** Property initializer */
        primary?: ValueOrObservable<boolean>;
        /** Property initializer */
        secondary?: ValueOrObservable<boolean>;
        /** Property initializer */
        dropdown?: ValueOrObservable<Menu.Option[]>;
        /** Property initializer */
        popover?: ValueOrObservable<typeof Container>;
        /** Property initializer */
        popoverAbove?: ValueOrObservable<boolean>;
        /** Property initializer */
        popoverTitle?: ValueOrObservable<string> | TextLabelFactory;
        /** Property initializer */
        href?: ValueOrObservable<ActivityResourcePath | string>;
        /** Property initializer */
        activity?: Activity | typeof Activity;
        /** Property initializer */
        style_button?: ValueOrObservable<Style | Style.StyleSet>;
        /** Signal initializer: method name or handler */
        DropdownClicked?: string | ActionHandler<string | number, any>;
    }
}
/** Button control with Bootstrap "primary" class (shortcut for setting .primary on regular Button class) */
export declare class PrimaryButton extends Button {
    primary: boolean;
}
/** Button that switches between active and inactive when clicked */
export declare class ToggleButton extends Button {
    constructor(label?: string | TextLabelFactory, icon?: string);
}
/** Button control with Bootstrap btn-link class */
export declare class LinkButton extends Button {
    constructor(label?: string | TextLabelFactory, icon?: string);
}
/** Button control that is not decorated as a button */
export declare class TextButton extends Button {
    constructor(label?: string | TextLabelFactory, icon?: string);
}
/** Round button control (e.g. with an icon) */
export declare class RoundButton extends Button {
    constructor(label?: string | TextLabelFactory, icon?: string);
}
export default Button;
