import Async from "typescene-async";
import Style from "./Style";
import Component from "./Components/Component";
/** Static class that manages the available (fixed) screen space in the browser window */
export declare abstract class Screen {
    static SHADE_OPACITY: number;
    static SHADE_GRADIENT_CENTER: number;
    static SHADE_BLUR: number;
    static SHADE_TRANSITION_SEC: number;
    /** Promise that resolves to true when ready to display the UI */
    static ready: Async.Promise<boolean>;
    /** True if display stack is empty (observable) */
    static isBlank: boolean;
    /** Encapsulation of screen container style */
    static style: Style;
    /** Number of pixels available on horizontal axis (observable) */
    static width: number;
    /** Number of pixels available on vertical axis (observable) */
    static height: number;
    /** True if any components are displayed as modals (observable) */
    static isModalDisplayed: boolean;
    /** True if root width is less than EXTRASMALL_WIDTH; (screen root element also has class "UI-Screen-ExtraSmall") Note that this also implies that .isSmall is true */
    static isExtraSmall: boolean;
    /** True if horizontal resolution is less than SMALL_WIDTH; (screen root element also has class "UI-Screen-Small") */
    static isSmall: boolean;
    /** True if horizontal resolution is LARGE_WIDTH or above; (screen root element also has class "UI-Screen-Large") */
    static isLarge: boolean;
    /** True if horizontal resolution is EXTRALARGE_WIDTH or above; (screen root element also has class "UI-Screen-ExtraLarge") Note that this also implies that .isLarge is true */
    static isExtraLarge: boolean;
    /** "Extra small" horizontal root width in pixels, default 550 */
    static EXTRASMALL_WIDTH: number;
    /** "Small" horizontal root width in pixels, default 720 */
    static SMALL_WIDTH: number;
    /** "Large" horizontal root width in pixels, default 960 */
    static LARGE_WIDTH: number;
    /** "Extra large" horizontal root width in pixels, default 1200 */
    static EXTRALARGE_WIDTH: number;
    /** Display a component, does nothing if given component is already displayed; positioning parameters are used if given, otherwise component is centered or displayed full screen */
    static display(component: Component | PromiseLike<Component>, vAlign?: string, hAlign?: string, margin?: string, keyHandler?: (KeyboardEvent) => void): void;
    /** Display a modal component on top of other content, blocking input to layers below; does nothing if given component is already displayed; positioning parameters are used if given, otherwise component is centered or displayed full screen */
    static displayModal(component: Component | PromiseLike<Component>, onEsc?: () => void, shade?: boolean, vAlign?: string, hAlign?: string, margin?: string, noAnimation?: boolean, keyHandler?: (KeyboardEvent) => void): void;
    /** Returns true if given component is currently on screen */
    static isDisplayed(component: Component): boolean;
    /** Remove the given component (previously passed to display*), and stop observing its output; if second argument is true, does not immediately remove the DOM element but wait for next screen update (e.g. to display another component) */
    static remove(component: Component | PromiseLike<Component>, noDOMUpdate?: boolean): void;
    /** Remove modal component(s) on top; does not invoke onEsc handlers */
    static closeModal(closeAll?: boolean): void;
    /** Remove all components to clear screen; if second argument is true, does not immediately remove the DOM elements but waits for next screen update (e.g. to display another component) */
    static clear(noDOMUpdate?: boolean): void;
    /** Fade given component after a delay (ms) in given number of ms (default no delay and 500ms transition time); removes element from screen after animation completes; does not block interactions while element is animating; returns a promise that resolves to the component itself */
    static fade<T extends Component>(component: T, delay?: number, transition?: number): Async.Promise<T>;
    /** (Try to) block input on screen until given promise is resolved, OR until another component is displayed */
    static blockAll(promise: PromiseLike<any> | null): void;
    /** Add all components to the fixed screen element */
    private static _update(noDOMUpdate?);
    /** Helper to create an observer for given container, or a new container wrapped around given modal block/component */
    private static _layerElement(component, vAlign?, hAlign?, margin?);
    /** Fade in modal shade (and cancel previous fade out) */
    private static _fadeShadeIn();
    /** Fade out modal shade and remove it (unless put back by later update) */
    private static _fadeShadeOut();
    /** Static initializer called when the DOM is ready */
    private static _init(resolve);
    private static _pendingUpdate;
    private static _screen;
    private static _shade;
    private static _shadeFadeTimer;
    private static _stack;
    constructor();
}
export declare namespace Screen {
    /** Context/dropdown menu root coordinates */
    interface MenuCoords {
        /** X-coordinate */
        x: number;
        /** Y-coordinate */
        y: number;
    }
}
export default Screen;
