import Async from "typescene-async";
import ComponentRenderOutput from "./Components/ComponentRenderOutput";
/** Encapsulates DOM element CSS styles, classes, and hidden attribute */
export declare class Style {
    /** Add global CSS styles for given selectors */
    static applyGlobalCSS(selectors: {
        [selector: string]: Style.StyleSet | Style;
    }, important?: boolean): void;
    /** Load external stylesheet(s) by URL; returns a promise that resolves after the style sheet(s) have been applied (or after a 1s timeout for browsers that do not support onload event for link elements) */
    static loadExternalCSS(...urls: string[]): PromiseLike<any>;
    /** Set the global root-em unit size (default overridden at 16px, unless in turn overridden by e.g. Bootstrap loaded through .loadExternalCSS(...)) */
    static setCSSRemSize(px: number): void;
    /** Create a new instance with given styles */
    constructor(styles?: Style.StyleSet, className?: string, hidden?: boolean);
    /** Add the given class name(s); returns this */
    addClass(className: string): this;
    /** Remove the given class name or any class names that match the given regular expression pattern (only if set directly on this Style instance, does not work on overrides); returns this */
    removeClass(className: string | RegExp): this;
    /** Remove (all) existing CSS class name(s) and use given class(es) */
    setClass(className: string): void;
    /** Return explicitly set value for given property */
    get(propertyName: string): any;
    /** Add the "hidden" attribute; returns this */
    hide(): this;
    /** Remove the "hidden" attribute; returns this */
    show(): this;
    /** Set a style property; if given value is observable, styles will also be applied on change (but does not watch directly); returns this */
    set(cssProperty: string, value: string | Async.ObservableValue<string>): any;
    /** Set multiple style properties using values in given object in alphabetical order (e.g. marginTop after margin); if given object is an Observable, styles will also be applied on change (but does not watch directly); returns this */
    set(obj: Style.StyleSet | Async.ObservableObject): any;
    /** Override styles and classes with those from given Style instance; returns this */
    override(style: Style | Style.StyleSet | Async.ObservableObject | Async.ObservableValue<Style | Style.StyleSet | Async.ObservableObject>): this;
    /** Apply style properties to given output element; returns this (does not observe styles afterwards) */
    applyTo(rendered: ComponentRenderOutput<any> | HTMLElement): void;
    /** Apply style properties to given output element and re-apply on change (to be used in async getter; works while getter is watched, e.g. for use in UI Component render function); returns this */
    applyToAsync(out: ComponentRenderOutput<any> | HTMLElement): this;
    private _UID;
    /** Set to 2 if non-empty styles/classes/hidden attrs were added, set to 1 if only empty styles were added */
    private _status;
    private _cssProps;
    private _sortedProps;
    private _className;
    private _hidden;
    private _isOverridden;
    private _overrides;
    private _observables;
    private _lastAppliedTo;
    private _lastOverrides;
    private _lastCSSText;
}
export declare namespace Style {
    /** A set of styles */
    interface StyleSet {
        [cssProperty: string]: string;
    }
}
export declare namespace Style {
    /** A style set for a tall drop card shadow */
    var DROPCARD_UP_SHADOW: Style.StyleSet;
}
export declare namespace Style {
    /** A style set for a drop card shadow */
    var DROPCARD_SHADOW: Style.StyleSet;
}
export declare namespace Style {
    /** A style set for a closer drop card */
    var DROPCARD_UP: Style.StyleSet;
}
export declare namespace Style {
    /** A style set for a drop card */
    var DROPCARD: Style.StyleSet;
}
export default Style;
